@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FopFactory.java 1762060 2016-09-23 12:57:46Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.apps;
 
@@ -24,13 +24,14 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.net.URI;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
 
 import org.xml.sax.SAXException;
 
-import org.apache.avalon.framework.configuration.Configuration;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -41,9 +42,11 @@ import org.apache.xmlgraphics.util.UnitConv;
 
 import org.apache.fop.apps.io.InternalResourceResolver;
 import org.apache.fop.apps.io.ResourceResolverFactory;
+import org.apache.fop.configuration.Configuration;
 import org.apache.fop.fo.ElementMapping;
 import org.apache.fop.fo.ElementMappingRegistry;
 import org.apache.fop.fonts.FontManager;
+import org.apache.fop.hyphenation.HyphenationTreeCache;
 import org.apache.fop.layoutmgr.LayoutManagerMaker;
 import org.apache.fop.render.ImageHandlerRegistry;
 import org.apache.fop.render.RendererConfig;
@@ -88,6 +91,8 @@ public final class FopFactory implements ImageContext {
 
     private final Map<String, RendererConfig> rendererConfig;
 
+    private HyphenationTreeCache hyphenationTreeCache;
+
     private FopFactory(FopFactoryConfig config) {
         this.config = config;
         this.resolver = ResourceResolverFactory.createInternalResourceResolver(config.getBaseURI(),
@@ -139,8 +144,14 @@ public final class FopFactory implements ImageContext {
      * @param baseURI the base URI to resolve resource URIs against
      * @return the requested FopFactory instance.
      */
-    public static FopFactory newInstance(URI baseURI) {
-        return new FopFactoryBuilder(baseURI).build();
+    public static FopFactory newInstance(final URI baseURI) {
+        return AccessController.doPrivileged(
+            new PrivilegedAction<FopFactory>() {
+                public FopFactory run() {
+                    return new FopFactoryBuilder(baseURI).build();
+                }
+            }
+        );
     }
 
     /**
@@ -163,7 +174,6 @@ public final class FopFactory implements ImageContext {
      * are particular to a rendering run. Don't reuse instances over multiple rendering runs but
      * instead create a new one each time and reuse the FopFactory.
      * @return the newly created FOUserAgent instance initialized with default values
-     * @throws FOPException
      */
     public FOUserAgent newFOUserAgent() {
         FOUserAgent userAgent = new FOUserAgent(this, resolver);
@@ -205,6 +215,18 @@ public final class FopFactory implements ImageContext {
         return userAgent.newFop(outputFormat, null);
     }
 
+    boolean isTableBorderOverpaint() {
+        return config.isTableBorderOverpaint();
+    }
+
+    boolean isSimpleLineBreaking() {
+        return config.isSimpleLineBreaking();
+    }
+
+    boolean isSkipPagePositionOnlyAllowed() {
+        return config.isSkipPagePositionOnlyAllowed();
+    }
+
     /**
      * Returns a new {@link Fop} instance. FOP will be configured with a default user agent
      * instance. Use this factory method if your output type requires an output stream.
@@ -326,6 +348,10 @@ public final class FopFactory implements ImageContext {
         return config.isAccessibilityEnabled();
     }
 
+    boolean isKeepEmptyTags() {
+        return config.isKeepEmptyTags();
+    }
+
     /** @see FopFactoryConfig#getImageManager() */
     public ImageManager getImageManager() {
         return config.getImageManager();
@@ -437,4 +463,11 @@ public final class FopFactory implements ImageContext {
     public ColorSpaceCache getColorSpaceCache() {
         return this.colorSpaceCache;
     }
+
+    public HyphenationTreeCache getHyphenationTreeCache() {
+        if (hyphenationTreeCache == null) {
+            hyphenationTreeCache = new HyphenationTreeCache();
+        }
+        return hyphenationTreeCache;
+    }
 }
