@@ -26,7 +26,7 @@ import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
 
-import org.apache.avalon.framework.configuration.Configuration;
+import org.xml.sax.SAXException;
 
 import org.apache.xmlgraphics.image.loader.ImageContext;
 import org.apache.xmlgraphics.image.loader.ImageManager;
@@ -35,6 +35,7 @@ import org.apache.xmlgraphics.io.ResourceResolver;
 
 import org.apache.fop.apps.io.InternalResourceResolver;
 import org.apache.fop.apps.io.ResourceResolverFactory;
+import org.apache.fop.configuration.Configuration;
 import org.apache.fop.fonts.FontManager;
 import org.apache.fop.layoutmgr.LayoutManagerMaker;
 
@@ -63,7 +64,7 @@ public final class FopFactoryBuilder {
     /**
      * A builder class for {@link FopFactory} which can be used for setting configuration. This is
      * a helper constructor that uses the default URI resolver implementation that FOP packages
-     * provide ({@link org.apache.fop.apps.io.ResourceResolverFactory.DefaultResourceResolver}).
+     * provide.
      *
      * @param defaultBaseURI the default base URI for resolving URIs against
      */
@@ -161,6 +162,11 @@ public final class FopFactoryBuilder {
         return this;
     }
 
+    public FopFactoryBuilder setKeepEmptyTags(boolean b) {
+        fopFactoryConfigBuilder.setKeepEmptyTags(b);
+        return this;
+    }
+
     /**
      * Sets the {@link LayoutManagerMaker} so that users can configure how FOP creates
      * {@link org.apache.fop.layoutmgr.LayoutManager}s.
@@ -297,7 +303,18 @@ public final class FopFactoryBuilder {
      * @return <code>this</code>
      */
     public FopFactoryBuilder setConfiguration(Configuration cfg) {
+        return setConfiguration(cfg, true);
+    }
+
+    protected FopFactoryBuilder setConfiguration(Configuration cfg, boolean parse) {
         fopFactoryConfigBuilder.setConfiguration(cfg);
+        if (parse) {
+            try {
+                new FopConfParser(cfg, this);
+            } catch (SAXException e) {
+                throw new RuntimeException(e);
+            }
+        }
         return this;
     }
 
@@ -323,6 +340,21 @@ public final class FopFactoryBuilder {
         return this;
     }
 
+    public FopFactoryBuilder setTableBorderOverpaint(boolean b) {
+        fopFactoryConfigBuilder.setTableBorderOverpaint(b);
+        return this;
+    }
+
+    public FopFactoryBuilder setSimpleLineBreaking(boolean b) {
+        fopFactoryConfigBuilder.setSimpleLineBreaking(b);
+        return this;
+    }
+
+    public FopFactoryBuilder setSkipPagePositionOnlyAllowed(boolean b) {
+        fopFactoryConfigBuilder.setSkipPagePositionOnlyAllowed(b);
+        return this;
+    }
+
     public static class FopFactoryConfigImpl implements FopFactoryConfig {
 
         private final EnvironmentProfile enviro;
@@ -331,6 +363,8 @@ public final class FopFactoryBuilder {
 
         private boolean accessibility;
 
+        private boolean keepEmptyTags = true;
+
         private LayoutManagerMaker layoutManagerMaker;
 
         private URI baseURI;
@@ -362,6 +396,11 @@ public final class FopFactoryBuilder {
 
         private Map<String, String> hyphPatNames;
 
+        private boolean tableBorderOverpaint;
+        private boolean simpleLineBreaking;
+
+        private boolean skipPagePositionOnlyAllowed = true;
+
         private static final class ImageContextImpl implements ImageContext {
 
             private final FopFactoryConfig config;
@@ -386,6 +425,10 @@ public final class FopFactoryBuilder {
             return accessibility;
         }
 
+        public boolean isKeepEmptyTags() {
+            return keepEmptyTags;
+        }
+
         /** {@inheritDoc} */
         public LayoutManagerMaker getLayoutManagerMakerOverride() {
             return layoutManagerMaker;
@@ -474,6 +517,18 @@ public final class FopFactoryBuilder {
             return isComplexScript;
         }
 
+        public boolean isTableBorderOverpaint() {
+            return tableBorderOverpaint;
+        }
+
+        public boolean isSimpleLineBreaking() {
+            return simpleLineBreaking;
+        }
+
+        public boolean isSkipPagePositionOnlyAllowed() {
+            return skipPagePositionOnlyAllowed;
+        }
+
         public Map<String, String> getHyphenationPatternNames() {
             return hyphPatNames;
         }
@@ -486,6 +541,7 @@ public final class FopFactoryBuilder {
     private interface FopFactoryConfigBuilder {
 
         void setAccessibility(boolean enableAccessibility);
+        void setKeepEmptyTags(boolean b);
 
         void setLayoutManagerMakerOverride(LayoutManagerMaker lmMaker);
 
@@ -518,6 +574,12 @@ public final class FopFactoryBuilder {
         void setComplexScriptFeaturesEnabled(boolean csf);
 
         void setHyphPatNames(Map<String, String> hyphPatNames);
+
+        void setTableBorderOverpaint(boolean b);
+
+        void setSimpleLineBreaking(boolean b);
+
+        void setSkipPagePositionOnlyAllowed(boolean b);
     }
 
     private static final class CompletedFopFactoryConfigBuilder implements FopFactoryConfigBuilder {
@@ -532,6 +594,9 @@ public final class FopFactoryBuilder {
         public void setAccessibility(boolean enableAccessibility) {
             throwIllegalStateException();
         }
+        public void setKeepEmptyTags(boolean b) {
+            throwIllegalStateException();
+        }
 
         public void setLayoutManagerMakerOverride(LayoutManagerMaker lmMaker) {
             throwIllegalStateException();
@@ -599,6 +664,17 @@ public final class FopFactoryBuilder {
             throwIllegalStateException();
         }
 
+        public void setTableBorderOverpaint(boolean b) {
+            throwIllegalStateException();
+        }
+
+        public void setSimpleLineBreaking(boolean b) {
+            throwIllegalStateException();
+        }
+
+        public void setSkipPagePositionOnlyAllowed(boolean b) {
+            throwIllegalStateException();
+        }
     }
 
     private static final class ActiveFopFactoryConfigBuilder implements FopFactoryConfigBuilder {
@@ -613,6 +689,10 @@ public final class FopFactoryBuilder {
             config.accessibility = enableAccessibility;
         }
 
+        public void setKeepEmptyTags(boolean b) {
+            config.keepEmptyTags = b;
+        }
+
         public void setLayoutManagerMakerOverride(LayoutManagerMaker lmMaker) {
             config.layoutManagerMaker = lmMaker;
         }
@@ -679,6 +759,18 @@ public final class FopFactoryBuilder {
         public void setHyphPatNames(Map<String, String> hyphPatNames) {
             config.hyphPatNames = hyphPatNames;
         }
+
+        public void setTableBorderOverpaint(boolean b) {
+            config.tableBorderOverpaint = b;
+        }
+
+        public void setSimpleLineBreaking(boolean b) {
+            config.simpleLineBreaking = b;
+        }
+
+        public void setSkipPagePositionOnlyAllowed(boolean b) {
+            config.skipPagePositionOnlyAllowed = b;
+        }
     }
 
 }
