@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PageBreaker.java 1768702 2016-11-08 14:34:28Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.layoutmgr;
 
@@ -68,6 +68,7 @@ public class PageBreaker extends AbstractBreaker {
     private MinOptMax relayedFootnoteSeparatorLength;
     private int previousFootnoteListIndex = -2;
     private int previousFootnoteElementIndex = -2;
+    private int prevousColumnCount;
 
     /**
      * The FlowLayoutManager object, which processes
@@ -146,7 +147,7 @@ public class PageBreaker extends AbstractBreaker {
 
         }
         if (needColumnBalancing) {
-            AbstractBreaker.log.debug(
+            log.debug(
                     "Column balancing necessary for the next element list!!!");
         }
         return nextSequenceStartsOn;
@@ -401,7 +402,7 @@ public class PageBreaker extends AbstractBreaker {
             }
         }
 
-        AbstractBreaker.log.debug("Restarting at " + restartPoint
+        log.debug("Restarting at " + restartPoint
                 + ", new start position: " + newStartPos);
 
         pageBreakHandled = true;
@@ -416,8 +417,8 @@ public class PageBreaker extends AbstractBreaker {
 
         PageBreakingAlgorithm algRestart;
         if (needColumnBalancing) {
-            AbstractBreaker.log.debug("Column balancing now!!!");
-            AbstractBreaker.log.debug("===================================================");
+            log.debug("Column balancing now!!!");
+            log.debug("===================================================");
 
             //Restart last page
             algRestart = new BalancingColumnBreakingAlgorithm(
@@ -425,7 +426,7 @@ public class PageBreaker extends AbstractBreaker {
                     alignment, Constants.EN_START, footnoteSeparatorLength,
                     isPartOverflowRecoveryActivated(),
                     pslm.getCurrentPV().getBodyRegion().getColumnCount());
-            AbstractBreaker.log.debug("===================================================");
+            log.debug("===================================================");
         } else  {
             // Handle special page-master for last page
             BodyRegion currentBody = pageProvider.getPage(false, currentPageNum)
@@ -436,22 +437,24 @@ public class PageBreaker extends AbstractBreaker {
             BodyRegion lastBody = pageProvider.getPage(false, currentPageNum)
                     .getPageViewport().getBodyRegion();
             lastBody.getMainReference().setSpans(currentBody.getMainReference().getSpans());
-            AbstractBreaker.log.debug("Last page handling now!!!");
-            AbstractBreaker.log.debug("===================================================");
+            log.debug("Last page handling now!!!");
+            log.debug("===================================================");
             //Restart last page
             algRestart = new PageBreakingAlgorithm(
                     getTopLevelLM(), getPageProvider(), createLayoutListener(),
                     alg.getAlignment(), alg.getAlignmentLast(),
                     footnoteSeparatorLength,
                     isPartOverflowRecoveryActivated(), false, false);
-            AbstractBreaker.log.debug("===================================================");
+            log.debug("===================================================");
         }
 
         int optimalPageCount = algRestart.findBreakingPoints(effectiveList,
                     newStartPos,
                     1, true, BreakingAlgorithm.ALL_BREAKS);
-        AbstractBreaker.log.debug("restart: optimalPageCount= " + optimalPageCount
-                + " pageBreaks.size()= " + algRestart.getPageBreaks().size());
+        if (algRestart.getPageBreaks() != null) {
+            log.debug("restart: optimalPageCount= " + optimalPageCount
+                    + " pageBreaks.size()= " + algRestart.getPageBreaks().size());
+        }
 
         boolean fitsOnePage
             = optimalPageCount <= pslm.getCurrentPV()
@@ -459,7 +462,7 @@ public class PageBreaker extends AbstractBreaker {
 
         if (needColumnBalancing) {
             if (!fitsOnePage) {
-                AbstractBreaker.log.warn(
+                log.warn(
                         "Breaking algorithm produced more columns than are available.");
                 /* reenable when everything works
                 throw new IllegalStateException(
@@ -467,16 +470,19 @@ public class PageBreaker extends AbstractBreaker {
                 */
             }
         } else {
-            if (fitsOnePage) {
+            boolean ipdChange = algRestart.getIPDdifference() != 0;
+            if (fitsOnePage && !ipdChange) {
                 //Replace last page
                 pslm.setCurrentPage(pageProvider.getPage(false, currentPageNum));
             } else {
                 //Last page-master cannot hold the content.
                 //Add areas now...
                 addAreas(alg, restartPoint, partCount - restartPoint, originalList, effectiveList);
-                //...and add a blank last page
-                setLastPageIndex(currentPageNum + 1);
-                pslm.setCurrentPage(pslm.makeNewPage(true));
+                if (!ipdChange) {
+                    //...and add a blank last page
+                    setLastPageIndex(currentPageNum + 1);
+                    pslm.setCurrentPage(pslm.makeNewPage(true));
+                }
                 return;
             }
         }
@@ -490,8 +496,8 @@ public class PageBreaker extends AbstractBreaker {
     }
 
     /** {@inheritDoc} */
-    protected void startPart(BlockSequence list, int breakClass) {
-        AbstractBreaker.log.debug("startPart() breakClass=" + getBreakClassName(breakClass));
+    protected void startPart(BlockSequence list, int breakClass, boolean emptyContent) {
+        log.debug("startPart() breakClass=" + getBreakClassName(breakClass));
         if (pslm.getCurrentPage() == null) {
             throw new IllegalStateException("curPage must not be null");
         }
@@ -505,7 +511,7 @@ public class PageBreaker extends AbstractBreaker {
                 // the current BlockSequence, it could have a break
                 // condition that must be satisfied;
                 // otherwise, we may simply need a new page
-                handleBreakTrait(breakClass);
+                handleBreakTrait(breakClass, emptyContent);
             }
             pageProvider.setStartOfNextElementList(pslm.getCurrentPageNum(),
                     pslm.getCurrentPV().getCurrentSpan().getCurrentFlowIndex(),
@@ -585,6 +591,10 @@ public class PageBreaker extends AbstractBreaker {
      * @param breakVal - value of break-before or break-after trait.
      */
     private void handleBreakTrait(int breakVal) {
+        handleBreakTrait(breakVal, false);
+    }
+
+    private void handleBreakTrait(int breakVal, boolean emptyContent) {
         Page curPage = pslm.getCurrentPage();
         switch (breakVal) {
         case Constants.EN_ALL:
@@ -623,7 +633,7 @@ public class PageBreaker extends AbstractBreaker {
                         pv.getCurrentSpan().moveToNextFlow();
                     } else {
                         log.trace("Making new page");
-                        pslm.makeNewPage(false);
+                        pslm.makeNewPage(false, emptyContent);
                     }
                 }
             }
@@ -700,16 +710,20 @@ public class PageBreaker extends AbstractBreaker {
         return layoutRedone;
     }
 
-    protected boolean lastPageHasIPDChange() {
+    protected boolean lastPageHasIPDChange(int optimalPageCount) {
         boolean lastPageMasterDefined = pslm.getPageSequence().hasPagePositionLast();
         boolean onlyPageMasterDefined = pslm.getPageSequence().hasPagePositionOnly();
         if (lastPageMasterDefined && !onlyPageMasterDefined) {
             // code not very robust and unable to handle situations were only and last are defined
+            int currentColumnCount = pageProvider.getCurrentColumnCount();
+            boolean changeInColumnCount = prevousColumnCount > 0 && prevousColumnCount != currentColumnCount;
+            prevousColumnCount = currentColumnCount;
+            if ((currentColumnCount > 1 && optimalPageCount % currentColumnCount == 0) || changeInColumnCount) {
+                return false;
+            }
             int currentIPD = this.pageProvider.getCurrentIPD();
             int lastPageIPD = this.pageProvider.getLastPageIPD();
-            if (lastPageIPD != -1 && currentIPD != lastPageIPD) {
-                return true;
-            }
+            return lastPageIPD != -1 && currentIPD != lastPageIPD;
         }
         return false;
     }
@@ -835,7 +849,7 @@ public class PageBreaker extends AbstractBreaker {
                 + ", break at pos " + endElementIndex + ", break class = "
                 + getBreakClassName(lastBreakClass));
 
-        startPart(effectiveList, lastBreakClass);
+        startPart(effectiveList, lastBreakClass, false);
 
         int displayAlign = getCurrentDisplayAlign();
 
