@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TableCellLayoutManager.java 1761020 2016-09-16 11:17:35Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -471,17 +471,30 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager {
                     BorderInfo border = gridUnits[0].getBorderStart();
                     int borderWidth = border.getRetainedWidth() / 2;
                     if (borderWidth > 0) {
-                        addBorder(blocks, y, 0, Trait.BORDER_START, border,
-                                  inFirstColumn, level);
-                        adjustXOffset(blocks[y][0], borderWidth);
-                        adjustIPD(blocks[y][0], -borderWidth);
+                        if (level == 1) {
+                            addBorder(blocks, y, gridUnits.length - 1, Trait.BORDER_START, border,
+                                      inFirstColumn, level);
+                            adjustIPD(blocks[y][gridUnits.length - 1], -borderWidth);
+                        } else {
+                            addBorder(blocks, y, 0, Trait.BORDER_START, border,
+                                      inFirstColumn, level);
+                            adjustXOffset(blocks[y][0], borderWidth);
+                            adjustIPD(blocks[y][0], -borderWidth);
+                        }
                     }
                     border = gridUnits[gridUnits.length - 1].getBorderEnd();
                     borderWidth = border.getRetainedWidth() / 2;
                     if (borderWidth > 0) {
-                        addBorder(blocks, y, gridUnits.length - 1, Trait.BORDER_END, border,
-                                  inLastColumn, level);
-                        adjustIPD(blocks[y][gridUnits.length - 1], -borderWidth);
+                        if (level == 1) {
+                            addBorder(blocks, y, 0, Trait.BORDER_END, border,
+                                      inLastColumn, level);
+                            adjustXOffset(blocks[y][0], borderWidth);
+                            adjustIPD(blocks[y][0], -borderWidth);
+                        } else {
+                            addBorder(blocks, y, gridUnits.length - 1, Trait.BORDER_END, border,
+                                    inLastColumn, level);
+                            adjustIPD(blocks[y][gridUnits.length - 1], -borderWidth);
+                        }
                     }
                 }
                 int dy = yoffset;
@@ -516,11 +529,13 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager {
         if (usedBPD < cellBPD) {
             if (getTableCell().getDisplayAlign() == EN_CENTER) {
                 Block space = new Block();
+                space.setChangeBarList(getChangeBarList());
                 space.setBPD((cellBPD - usedBPD) / 2);
                 space.setBidiLevel(getTableCell().getBidiLevelRecursive());
                 curBlockArea.addBlock(space);
             } else if (getTableCell().getDisplayAlign() == EN_AFTER) {
                 Block space = new Block();
+                space.setChangeBarList(getChangeBarList());
                 space.setBPD(cellBPD - usedBPD);
                 space.setBidiLevel(getTableCell().getBidiLevelRecursive());
                 curBlockArea.addBlock(space);
@@ -607,6 +622,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager {
                            boolean outer, int level) {
         if (blocks[i][j] == null) {
             blocks[i][j] = new Block();
+            blocks[i][j].setChangeBarList(getChangeBarList());
             blocks[i][j].addTrait(Trait.IS_REFERENCE_AREA, Boolean.TRUE);
             blocks[i][j].setPositioning(Block.ABSOLUTE);
             blocks[i][j].setBidiLevel(level);
@@ -638,6 +654,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager {
         int paddingEnd = padding.getPaddingEnd(false, this);
 
         Block block = new Block();
+        block.setChangeBarList(getChangeBarList());
         TraitSetter.setProducerID(block, getTable().getId());
         block.setPositioning(Block.ABSOLUTE);
         block.setIPD(cellIPD + paddingStart + paddingEnd);
@@ -664,6 +681,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager {
     public Area getParentArea(Area childArea) {
         if (curBlockArea == null) {
             curBlockArea = new Block();
+            curBlockArea.setChangeBarList(getChangeBarList());
             curBlockArea.addTrait(Trait.IS_REFERENCE_AREA, Boolean.TRUE);
             TraitSetter.setProducerID(curBlockArea, getTableCell().getId());
             curBlockArea.setPositioning(Block.ABSOLUTE);
@@ -685,7 +703,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager {
      * @param childArea the child to add to the cell
      */
     public void addChildArea(Area childArea) {
-        if (curBlockArea != null) {
+        if (curBlockArea != null && childArea instanceof Block) {
             curBlockArea.addBlock((Block) childArea);
         }
     }
